#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "graphics.h"
#include "room.h"
#include "globe.h"
#include "version.h"


int
main(
	int   argc,
	char *argv[]
)
{
	printf("Version: %s\n", VERSION);

	//////////

	srand(time(nullptr));

	GRAPHICS graphics(640,480);
	ROOM room(&graphics);
	GLOBE globe(&graphics);

	int quit = 0;
	int pause = 0;

	int drag_x = -1;
	int drag_y = -1;
	int shaken = 0;
	int reaching = 0;

	while (!quit) {
		SDL_Event ev;

		while (SDL_PollEvent(&ev)) {
			if (ev.type == SDL_KEYDOWN) {
				if (ev.key.keysym.sym == SDLK_ESCAPE) {
					quit = 1;
				}
				else if (ev.key.keysym.sym == SDLK_SPACE) {
					pause = 1 - pause;
				}
			}
			else if (ev.type == SDL_MOUSEBUTTONDOWN) {
				if (ev.button.button == 1) {
					reaching = 1;
					globe.hand_start();

					SDL_GetMouseState(&drag_x, &drag_y);

					shaken = 0;
				}
			}
			else if (ev.type == SDL_MOUSEBUTTONUP) {
				if (ev.button.button == 1) {
					reaching = 0;
					globe.hand_stop();

					int tmp_mx;
					int tmp_my;
					SDL_GetMouseState(&tmp_mx, &tmp_my);

					if (!shaken) {
					   // click
					   printf("click\n");
					}
				}
			}
			else if (ev.type == SDL_MOUSEMOTION) {

				if (globe.full_hand()) {
					shaken = 1;

					globe.shake(ev.motion.x - drag_x, ev.motion.y - drag_y);

					drag_x = ev.motion.x;
					drag_y = ev.motion.y;
				}
				else if (reaching) {
					// ignore motion, until we've grabbed it
					SDL_GetMouseState(&drag_x, &drag_y);
				}
			}
		}

		//////////
		// update

		if (!pause) {
			// FIXME: calc real update time
			globe.update(1.0/30);
		}

		//////////
		// draw

		room.draw();
		globe.draw();

		//////////
		// flip & wait
		SDL_Flip(graphics.primary);

		usleep(1000000 / 30);
	}

	return(0);
}



