#include "utils.h"
#include "world.h"

#define EVOLVE_CHECK(F) (RANDI((int) (((double) F) * 30.0 / dt)) == 0)

#define ES2DX(s) ((s == ES_MOVE_EAST) ? 1 : \
                  (s == ES_MOVE_WEST) ? -1  :  0)
#define ES2DY(s) ((s == ES_MOVE_NORTH) ? 1 : \
                  (s == ES_MOVE_SOUTH) ? -1  :  0)

////////////////////////////////////////

void
CELL::reserve_if_possible(
	WORLD *w
)
{
	;
}

////////////////////////////////////////

WORLD::WORLD(
	GRAPHICS *graphics
)
{
	_graphics = graphics;

	_world_surf = _graphics->load_image("resources/world.png");
	_tree_surf  = _graphics->load_image("resources/tree.png");
	_deer_surf  = _graphics->load_image("resources/deer.png");
	_man_surf   = _graphics->load_image("resources/man.png");
	_man_hunt_surf = _graphics->load_image("resources/man_hunt.png");

	//////////

	_side_view = 1;

	_shaking = 0;
	_shake_time = 0;

	//////////

	int x;
	int y;

	for(x = 0; x < 11; x++) {
		for(y = 0; y < 11; y++) {
			grid[x][y].state = -1;
			grid[x][y].size = -1;
			grid[x][y].ent = ENT_NONE;
			grid[x][y].ent_state = ES_STOPPED;
			grid[x][y].ent_frame = 0;
			grid[x][y].ent_dir   = ES_STOPPED; // not really meaningful, hopefully everyone else is keeping it straight.

			if ((((x == 0) || (x == 10)) && (y != 5)) ||
			    (((y == 0) || (y == 10)) && (x != 5))) {
				grid[x][y].type = CELL_OOB;
			}
			else if ((((x <= 2) || (x >= 8)) && ((y == 1) || (y == 9))) ||
			         (((y <= 2) || (y >= 8)) && ((x == 1) || (x == 9)))) {
				grid[x][y].type = CELL_OOB;
			}
			else {
				grid[x][y].type = CELL_EMPTY;
			}
		}
	}

#ifndef RELEASE
	dump_grid();
#endif
}

WORLD::~WORLD(
)
{
	;
}

//////////////////////////////////////////////////////////////////////

void 
WORLD::update(
	float dt
)
{
	static float time = 0.0;
	int x;
	int y;

	//////////

	time += dt;

	//////////

	int grass_count = 0;
	int tree_count = 0;
	int deer_count = 0;
	int man_count = 0;

	// Count stuff, for later
	for(x = 0; x < 11; x++) {
		for(y = 0; y < 11; y++) {
			if (grid[x][y].type == CELL_GRASS) {
				grass_count++;
			}
			else if (grid[x][y].type == CELL_TREE) {
				tree_count++;
			}

			if (grid[x][y].ent == ENT_DEER) {
				deer_count++;
			}
			else if (grid[x][y].ent == ENT_MAN) {
				man_count++;
			}
		}
	}

	//////////

	int add_deer;
	int add_man;

	add_deer = (tree_count+grass_count)/5 - deer_count;
	add_deer = (add_deer < 0) ? 0 : add_deer;

	add_man = (deer_count)/3 - man_count;
	add_man = (add_man < 0) ? 0 : add_man;

	//////////

	// update things

	for(x = 0; x < 11; x++) {
		for(y = 10; y >= 0; y--) {
			if (grid[x][y].type == CELL_EMPTY) {
				if (EVOLVE_CHECK(1)) {
					grid[x][y].type = CELL_GRASS;
				}
			}
			else if (grid[x][y].type == CELL_GRASS) {
				if ((add_deer > 0) &&
				    (grid[x][y].ent == ENT_NONE) &&
				    EVOLVE_CHECK(1)) {
					grid[x][y].ent = ENT_DEER;
					grid[x][y].ent_state = ES_STOPPED;
					grid[x][y].ent_dir = (ENT_STATE) (ES_MOVE_EAST + RANDI(4));
					grid[x][y].ent_frame = 0;
					grid[x][y].ent_offset_x = 0;
					grid[x][y].ent_offset_y = 0;
					add_deer--;
				}
				else if ((add_man > 0) &&
				         (grid[x][y].ent == ENT_NONE) &&
				         EVOLVE_CHECK(1)) {
					grid[x][y].ent = ENT_MAN;
					grid[x][y].ent_state = ES_STOPPED;
					grid[x][y].ent_dir = (ENT_STATE) (ES_MOVE_EAST + RANDI(4));
					grid[x][y].ent_frame = 0;
					grid[x][y].ent_offset_x = 0;
					grid[x][y].ent_offset_y = 0;
					add_man--;
				}

				if (EVOLVE_CHECK(1)) {
					grid[x][y].type = CELL_TREE;
					grid[x][y].size = 0;
					grid[x][y].state = 0;
					grid[x][y].offset = 8 - RANDI(16);
				}
			}
			else if (grid[x][y].type == CELL_TREE) {
				if ((add_deer > 0) &&
				    (grid[x][y].ent == ENT_NONE) &&
				    EVOLVE_CHECK(1)) {
					grid[x][y].ent = ENT_DEER;
					grid[x][y].ent_state = ES_STOPPED;
					grid[x][y].ent_dir = (ENT_STATE) (ES_MOVE_EAST + RANDI(4));
					grid[x][y].ent_frame = 0;
					grid[x][y].ent_offset_x = 0;
					grid[x][y].ent_offset_y = 0;
					add_deer--;
				}
				else if ((add_man > 0) &&
				         (grid[x][y].ent == ENT_NONE) &&
				         EVOLVE_CHECK(1)) {
					grid[x][y].ent = ENT_MAN;
					grid[x][y].ent_state = ES_STOPPED;
					grid[x][y].ent_dir = (ENT_STATE) (ES_MOVE_EAST + RANDI(4));
					grid[x][y].ent_frame = 0;
					grid[x][y].ent_offset_x = 0;
					grid[x][y].ent_offset_y = 0;
					add_man--;
				}

				//
				// state represents falling-over-ness
				//

				if (grid[x][y].state != 0) {
					if (RANDI(10) == 0) {
						int *s = &(grid[x][y].state);

						*s += (*s < 0) ? -1 : 1;

						if ((*s <= -3) || (*s >= 3)) {
							*s = (*s < 0) ? -3 : 3;

							if (RANDI(5) == 0) {
								// printf("Decay %d, %d\n", x, y);
								grid[x][y].size -= 1;

								if (grid[x][y].size == 0) {
									grid[x][y].type = CELL_GRASS;
								}
							}
						}
					}
				}

			   // You bastard, you killed it...
				if ((_shaking != 0) && (grid[x][y].size >= 0) &&
				    (RANDI(120 / (grid[x][y].size+1)) == 0)) {
					grid[x][y].state = _shaking;
				}

				if (EVOLVE_CHECK(2)) {
					if (grid[x][y].size < 2) {
						grid[x][y].size++;
					}
					else if (RANDI(1) == 0) {
						// printf("Killing %d, %d\n", x, y);
						// kill tree
						grid[x][y].state = RANDI(1)*2 - 1;
					}
				}
			}
		}
	}

	//////////

	// unshake things
	_shake_time += dt;

	if (_shake_time > .1) {
		_shake_time -= .1;

		if (_shaking < 0) { _shaking++; }
		if (_shaking > 0) { _shaking--; }
	}


	//////////

	// update ents

	for(x = 0; x < 11; x++) {
		for(y = 10; y >= 0; y--) {

			CELL *g = &(grid[x][y]);

			if ((g->ent == ENT_MAN) && (g->ent_state == ES_STOPPED)) {
				if ((x < 10) && (grid[x+1][y].ent == ENT_DEER)) {
					// printf("hunting from %d,%d -> %d,%d\n", x,y, x+1, y);
					g->ent_state = ES_HUNTING;
					g->ent_frame = 0;
					grid[x+1][y].ent_state = ES_HUNTED;
					grid[x+1][y].ent_frame = 0;
				}
			}

			if ((g->ent == ENT_DEER) || (g->ent == ENT_MAN)) {
				if ((g->ent_state == ES_STOPPED) && EVOLVE_CHECK(.5) ) {

					ENT_STATE new_dir = (ENT_STATE) (ES_MOVE_EAST + RANDI(4));
					int x2 = x + ES2DX(new_dir);
					int y2 = y + ES2DY(new_dir);

					// printf("attempting to move %d,%d -%d-> %d,%d: ", x,y, new_dir, x2,y2);

					if ((x2 >= 0) && (x2 < 10) &&
					    (y2 >= 0) && (y2 < 10) &&
					    (grid[x2][y2].type != CELL_OOB) &&
					    (grid[x2][y2].ent == ENT_NONE)) {

						grid[x2][y2].ent = ENT_RESERVED;

						g->ent_state = ES_MOVING;
						g->ent_dir   = new_dir;
						g->ent_frame = 0;
						g->ent_offset_x = 0;
						g->ent_offset_y = 0;
						// printf("ok\n");
					}
					else {
						// printf("denied\n");
					}

				}
				else if ((g->ent_state != ES_STOPPED) &&
				         (g->ent_state != ES_HUNTING) &&
				         (g->ent_state != ES_HUNTED)) {

					g->ent_offset_x += ES2DX(g->ent_dir);
					g->ent_offset_y += ES2DY(g->ent_dir);

					g->ent_frame += 8 * dt;
					if (g->ent_frame > 4) { g->ent_frame -= 4; }

					if ((g->ent_offset_x > 16) || (g->ent_offset_x < -16) ||
						 (g->ent_offset_y > 16) || (g->ent_offset_y < -16)) {

						int x2 = x + ES2DX(g->ent_dir);
						int y2 = y + ES2DY(g->ent_dir);

						// This shouldn't be violated, since we're checking earlier...
						if ((x2 >= 0) && (x2 < 11) &&
							 (y2 >= 0) && (y2 < 11) &&
							 (grid[x2][y2].type != CELL_OOB)) {

							// Moving to new cell
							grid[x2][y2].ent = g->ent;
							grid[x2][y2].ent_state = g->ent_state;
							grid[x2][y2].ent_dir   = g->ent_dir;
							grid[x2][y2].ent_frame = 0;
							grid[x2][y2].ent_offset_x = -g->ent_offset_x + ES2DX(g->ent_dir);
							grid[x2][y2].ent_offset_y = -g->ent_offset_y + ES2DY(g->ent_dir);

							g->ent = ENT_NONE;
						}
						else {
							// Should never occur
							printf("what?: %d, %d -%d-> %d, %d\n", x, y, g->ent_dir, x2, y2);
						}
					}

					if ((g->ent_offset_x == 0) && (g->ent_offset_y == 0)) {
						g->ent_state = ES_STOPPED;
						g->ent_frame = 0;
					}
				}
				else if ((g->ent == ENT_MAN) && (g->ent_state == ES_HUNTING)) {
					g->ent_frame += 8 * dt;
					if (g->ent_frame > 8) {
						g->ent_state = ES_STOPPED;
						g->ent_frame = 0;
						grid[x+1][y].ent = ENT_NONE;
					}
				}
			}
		}
	}

	//////////

	if (time > 5.0) {
		time -= 5.0;

#ifndef RELEASE
		dump_grid();
#endif
	}
}


void 
WORLD::draw(
	int offset_x,
	int offset_y
)
{
	if (_side_view) {
		draw_side(offset_x, offset_y);
	}
	else {
		draw_top(offset_x, offset_y);
	}
}



void 
WORLD::draw_side(
	int offset_x,
	int offset_y
)
{
	int pos_x = 120 + 16 + 8 + offset_x;
	int pos_y = 250 + offset_y;

	SDL_Rect srect = { x: 0, y: 0, w: 32, h:32 };
	SDL_Rect drect = { x: 0, y: 0, w: 32, h:32 };

	for(int y = 0; y < 11; y++) {
		for(int x = 0; x < 11; x++) {
			// background
			srect.h = 32;
			srect.w = 32;

			drect.x = pos_x + x * 32;
			drect.y = pos_y - 32;
			drect.h = 32;
	
			if (grid[x][y].type == CELL_GRASS) {
				srect.x = 0;
				srect.y = 0;
				SDL_BlitSurface(_world_surf, &srect, _graphics->primary, &drect);
			}
			else if (grid[x][y].type == CELL_TREE) {
#if 1
				if (grid[x][y].state != 0) {
					srect.x = (3 + grid[x][y].state) * 64;
				}
				else {
					srect.x = (3 + _shaking) * 64;
				}
				srect.y = grid[x][y].size * 64;
				srect.w = 64;
				srect.h = 64;
				drect.x += grid[x][y].offset - 32/2;
				drect.y = pos_y - 64;
				drect.w = srect.w;
				drect.h = srect.h;
				SDL_BlitSurface(_tree_surf, &srect, _graphics->primary, &drect);
#endif
			}

			// foreground
			if (grid[x][y].ent == ENT_DEER) {
				srect.x = ((int) grid[x][y].ent_frame) * 32;
				srect.y = (grid[x][y].ent_dir - 1) * 32;
				srect.h = 32;
				srect.w = 32;

				drect.x = pos_x + x * 32 + grid[x][y].ent_offset_x;
				drect.y = pos_y - 32;
				drect.h = 32;
				drect.w = 32;

				SDL_BlitSurface(_deer_surf, &srect, _graphics->primary, &drect);
			}
			else if (grid[x][y].ent == ENT_MAN) {
				if (grid[x][y].ent_state == ES_HUNTING) {
					int f = (int) grid[x][y].ent_frame;
					srect.x = (f % 2) * 64;
					srect.y = (f / 2) * 32; 
					srect.h = 32;
					srect.w = 64;

					drect.x = pos_x + x * 32 + grid[x][y].ent_offset_x;
					drect.y = pos_y - 32;
					drect.h = srect.h;
					drect.w = srect.w;

					SDL_BlitSurface(_man_hunt_surf, &srect, _graphics->primary, &drect);
				}
				else {
					srect.x = ((int) grid[x][y].ent_frame) * 32;
					srect.y = (grid[x][y].ent_dir - 1) * 32;
					srect.h = 32;
					srect.w = 32;

					drect.x = pos_x + x * 32 + grid[x][y].ent_offset_x;
					drect.y = pos_y - 32;
					drect.h = 32;
					drect.w = 32;

					SDL_BlitSurface(_man_surf, &srect, _graphics->primary, &drect);
				}
			}
		}
	}
}


void 
WORLD::draw_top(
	int offset_x,
	int offset_y
)
{
	;
}

void 
WORLD::shake(
	int dx,
	int dy
)
{
	if (_side_view) {
		_shake_time = 0;

		_shaking += (dx > 0) ? 1 :
		            (dx < 0) ? -1 : 0;
		if (_shaking < -2) { _shaking = -2; }
		if (_shaking >  2) { _shaking =  2; }
	}
	else {
		printf("shake: Unimplemented\n");
	}
}



void 
WORLD::dump_grid(
	void
)
{
	int x;
	int y;

	printf("----------\n");

	for(y = 0; y < 11; y++) {
		for(x = 0; x < 11; x++) {
			printf("% 2d%c ",
			       grid[x][y].type,
			       (grid[x][y].ent == ENT_DEER) ? 'd' :
			       (grid[x][y].ent == ENT_MAN) ? 'm' : ' ');
		}
		printf("\n");
	}
}


