#if !defined(INCLUDED_WORLD_H)
#define INCLUDED_WORLD_H

#include "graphics.h"

class WORLD;


typedef enum {
	CELL_OOB = -1,
	CELL_EMPTY = 0,
	CELL_GRASS = 1,
	CELL_TREE = 2,
} CELL_TYPE;

typedef enum {
	ENT_RESERVED = -2,
	ENT_NONE = -1,
	ENT_DEER =  0,
	ENT_MAN  =  1,
} ENT_TYPE;

typedef enum {
	ES_STOPPED = 0,
	ES_MOVING = 1,
	ES_HUNTING = 2,
	ES_HUNTED  = 2, // since deer can't actually hunt

	ES_MOVE_EAST = 1, // make sure these are sequential...
	ES_MOVE_WEST = 2,
	ES_MOVE_NORTH = 3,
	ES_MOVE_SOUTH = 4,
} ENT_STATE;

typedef struct Cell {
	CELL_TYPE type;
	int       size;
	int       state;

	int       offset;

	ENT_TYPE  ent;
	ENT_STATE ent_state;
	ENT_STATE ent_dir;
	double    ent_frame;
	int       ent_offset_x;
	int       ent_offset_y;

public:
	void reserve_if_possible( WORLD *w );

} CELL;


class WORLD {
public:
	WORLD(GRAPHICS *graphics);
	~WORLD();

	void update(float dt);

	void draw( int offset_x, int offset_y );
	void draw_side( int offset_x, int offset_y );
	void draw_top( int offset_x, int offset_y );

	//////////

	void shake(int dx, int dy);

private:
	void dump_grid(void);

	

private:
	GRAPHICS *_graphics;

	SDL_Surface *_world_surf;
	SDL_Surface *_tree_surf;
	SDL_Surface *_deer_surf;
	SDL_Surface *_man_surf;
	SDL_Surface *_man_hunt_surf;

	int _side_view;
	int _shaking;
	double _shake_time;

	CELL grid[11][11];
};


#else

class WORLD;

#endif
